@@ -2,7 +2,7 @@
 /*
  * Copyright (C) 2002-2003 CodeFactory AB
  * Copyright (C) 2001-2003 Mikael Hallendal <micke@imendio.com>
- * Copyright (C) 2004 Imendio hB
+ * Copyright (C) 2004-2005 Imendio hB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,8 +22,6 @@
 
 #include <stdio.h>
 #include <string.h>
-
-#include <libgnomevfs/gnome-vfs.h>
 #include <gtkmozembed.h>
 
 #include "dh-util.h"
@@ -38,21 +36,30 @@ struct _DhHtmlPriv {
 	GtkMozEmbed *gecko;
 };
 
-static void     html_class_init               (DhHtmlClass         *klass);
-static void     html_init                     (DhHtml              *html);
-static void     html_title_cb                 (GtkMozEmbed         *embed,
-					       DhHtml              *html);
-static void     html_location_cb              (GtkMozEmbed         *embed, 
-					       DhHtml              *html);
+static void     html_class_init  (DhHtmlClass *klass);
+static void     html_init        (DhHtml      *html);
+static void     html_title_cb    (GtkMozEmbed *embed,
+				  DhHtml      *html);
+static void     html_location_cb (GtkMozEmbed *embed,
+				  DhHtml      *html);
+static gboolean html_open_uri_cb (GtkMozEmbed *embed,
+				  const gchar *uri,
+				  DhHtml      *html);
+
 
 enum {
 	TITLE_CHANGED,
 	LOCATION_CHANGED,
+	OPEN_URI,
+	OPEN_NEW_TAB,
 	LAST_SIGNAL
 };
 
 static gint signals[LAST_SIGNAL] = { 0 };
 
+/* Has the value of the URL under the mouse pointer, otherwise NULL */
+static gchar *current_url = NULL;
+
 GType
 dh_html_get_type (void)
 {
@@ -102,6 +109,64 @@ html_class_init (DhHtmlClass *klass)
 			      dh_marshal_VOID__STRING,
 			      G_TYPE_NONE,
 			      1, G_TYPE_STRING);
+	signals[OPEN_URI] =
+		g_signal_new ("open-uri",
+			      G_TYPE_FROM_CLASS (klass),
+			      G_SIGNAL_RUN_LAST,
+			      0,
+			      NULL, NULL,
+			      dh_marshal_BOOLEAN__STRING,
+			      G_TYPE_BOOLEAN,
+			      1, G_TYPE_STRING);
+	signals[OPEN_NEW_TAB] =
+		g_signal_new ("open-new-tab",
+			      G_TYPE_FROM_CLASS (klass),
+			      G_SIGNAL_RUN_LAST,
+			      0,
+			      NULL, NULL,
+			      dh_marshal_VOID__STRING,
+			      G_TYPE_NONE,
+			      1, G_TYPE_STRING);
+}
+
+static gboolean
+html_mouse_click_cb (GtkMozEmbed *widget, gpointer dom_event, DhHtml *html)
+{
+	gint button;
+	gint mask;
+	
+	button = dh_gecko_utils_get_mouse_event_button (dom_event);
+	mask = dh_gecko_utils_get_mouse_event_modifiers (dom_event);
+
+	if (button == 2 || (button == 1 && mask & GDK_CONTROL_MASK)) {
+		if (current_url) {
+			g_signal_emit (html,
+				       signals[OPEN_NEW_TAB], 0,
+				       current_url);
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
+/* I'd like to get rid of this hack, there should be a way to get the URI that
+ * was clicked instead of tracking it like this.
+ */
+static gboolean
+html_link_message_cb (GtkMozEmbed *widget)
+{
+	if (current_url) {
+		g_free (current_url);
+	}
+	current_url = gtk_moz_embed_get_link_message (widget);
+
+	if (current_url[0] == '\0') {
+		g_free (current_url);
+		current_url = NULL;
+	}
+
+	return FALSE;
 }
 
 static void
@@ -138,8 +203,8 @@ html_init (DhHtml *html)
         DhHtmlPriv *priv;
         
 	priv = g_new0 (DhHtmlPriv, 1);
-
-	priv->gecko = (GtkMozEmbed *) gtk_moz_embed_new ();
+	
+	priv->gecko = GTK_MOZ_EMBED (gtk_moz_embed_new ());
 
 	g_signal_connect (priv->gecko, "title",
 			  G_CALLBACK (html_title_cb),
@@ -147,6 +212,15 @@ html_init (DhHtml *html)
 	g_signal_connect (priv->gecko, "location",
 			  G_CALLBACK (html_location_cb),
 			  html);
+	g_signal_connect (priv->gecko, "open-uri",
+			  G_CALLBACK (html_open_uri_cb),
+			  html);
+	g_signal_connect (priv->gecko, "dom_mouse_click",
+			  G_CALLBACK (html_mouse_click_cb),
+			  html);
+	g_signal_connect (priv->gecko, "link_message",
+			  G_CALLBACK (html_link_message_cb),
+			  html);
 	g_signal_connect (priv->gecko, "add",
 			  G_CALLBACK (html_child_add_cb),
 			  html);
@@ -154,7 +228,7 @@ html_init (DhHtml *html)
 			  G_CALLBACK (html_child_remove_cb),
 			  html);
 
-        html->priv = priv;
+	html->priv = priv;
 }
 
 static void
@@ -163,9 +237,7 @@ html_title_cb (GtkMozEmbed *embed, DhHtml *html)
 	char *new_title;
 
 	new_title = gtk_moz_embed_get_title (embed);
-	if (new_title && *new_title != '\0') {
-		g_signal_emit (html, signals[TITLE_CHANGED], 0, new_title);
-	}
+	g_signal_emit (html, signals[TITLE_CHANGED], 0, new_title);
 	g_free (new_title);
 }
 
@@ -182,13 +254,26 @@ html_location_cb (GtkMozEmbed *embed, DhHtml *html)
 	g_free (location);
 }
 
+static gboolean
+html_open_uri_cb (GtkMozEmbed *embed, const gchar *uri, DhHtml *html)
+{
+	DhHtmlPriv *priv;
+	gboolean   retval;
+
+	priv = html->priv;
+
+	retval = TRUE;
+
+	g_signal_emit (html, signals[OPEN_URI], 0, uri, &retval);
+
+	return retval;
+}
+
 DhHtml *
 dh_html_new (void)
 {
         DhHtml *html;
 
-        d(puts(__FUNCTION__));
-
         html = g_object_new (DH_TYPE_HTML, NULL);
  
         return html;
@@ -197,7 +282,7 @@ dh_html_new (void)
 void
 dh_html_clear (DhHtml *html)
 {
-	DhHtmlPriv  *priv;
+	DhHtmlPriv        *priv;
 	static const char *data = "<html><body bgcolor=\"white\"></body></html>";
 	
 	g_return_if_fail (DH_IS_HTML (html));
@@ -214,8 +299,6 @@ dh_html_open_uri (DhHtml *html, const gchar *str_uri)
         DhHtmlPriv *priv;
 	gchar      *full_uri;
 	
-        d(puts(__FUNCTION__));
-	
 	g_return_if_fail (DH_IS_HTML (html));
 	g_return_if_fail (str_uri != NULL);
 
@@ -237,11 +320,13 @@ dh_html_open_uri (DhHtml *html, const gchar *str_uri)
 GtkWidget *
 dh_html_get_widget (DhHtml *html)
 {
-	g_return_val_if_fail (DH_IS_HTML (html), NULL);
-	
-	gtk_widget_show (GTK_WIDGET (html->priv->gecko));
+	DhHtmlPriv *priv;
 
-	return GTK_WIDGET (html->priv->gecko);
+	g_return_val_if_fail (DH_IS_HTML (html), NULL);
+
+	priv = html->priv;
+
+	return GTK_WIDGET (priv->gecko);
 }
 
 gboolean
@@ -292,5 +377,38 @@ dh_html_go_back (DhHtml *html)
 	gtk_moz_embed_go_back (priv->gecko);
 }
 
+gchar *
+dh_html_get_title (DhHtml *html)
+{
+	DhHtmlPriv *priv;
+	
+	g_return_val_if_fail (DH_IS_HTML (html), NULL);
 
+	priv = html->priv;
+	
+	return gtk_moz_embed_get_title (priv->gecko);
+}
 
+gchar *
+dh_html_get_location (DhHtml *html)
+{
+	DhHtmlPriv *priv;
+	
+	g_return_val_if_fail (DH_IS_HTML (html), NULL);
+
+	priv = html->priv;
+	
+	return gtk_moz_embed_get_location (priv->gecko);
+}
+
+void
+dh_html_copy_selection (DhHtml *html)
+{
+	DhHtmlPriv *priv;
+	
+	g_return_if_fail (DH_IS_HTML (html));
+
+	priv = html->priv;
+
+	dh_gecko_utils_copy_selection (priv->gecko);
+}
