@@ -20,7 +20,6 @@
  */
 
 #include <gtk/gtktreemodel.h>
-#include <libgnome/gnome-i18n.h>
 #include <string.h>
 
 #include "dh-link.h"
@@ -43,6 +42,7 @@ static void     keyword_model_tree_model_init   (GtkTreeModelIface  *iface);
 
 
 static void     keyword_model_finalize          (GObject            *object);
+static GtkTreeModelFlags keyword_model_get_flags (GtkTreeModel *tree_model);
 static gint     keyword_model_get_n_columns     (GtkTreeModel       *tree_model);
 static GType    keyword_model_get_column_type   (GtkTreeModel       *tree_model,
 						 gint                keyword);
@@ -129,6 +129,7 @@ keyword_model_class_init (DhKeywordModelClass *class)
 static void
 keyword_model_tree_model_init (GtkTreeModelIface *iface)
 {
+	iface->get_flags       = keyword_model_get_flags;
         iface->get_n_columns   = keyword_model_get_n_columns;
         iface->get_column_type = keyword_model_get_column_type;
         iface->get_iter        = keyword_model_get_iter;
@@ -176,6 +177,12 @@ keyword_model_finalize (GObject *object)
         (* parent_class->finalize) (object);
 }
 
+static GtkTreeModelFlags
+keyword_model_get_flags (GtkTreeModel *tree_model)
+{
+	return GTK_TREE_MODEL_ITERS_PERSIST | GTK_TREE_MODEL_LIST_ONLY;
+}
+
 static gint
 keyword_model_get_n_columns (GtkTreeModel *tree_model)
 {
@@ -184,7 +191,7 @@ keyword_model_get_n_columns (GtkTreeModel *tree_model)
 
 static GType
 keyword_model_get_column_type (GtkTreeModel *tree_model,
-                     gint          column)
+			       gint          column)
 {
 	switch (column) {
 	case DH_KEYWORD_MODEL_COL_NAME:
@@ -200,13 +207,13 @@ keyword_model_get_column_type (GtkTreeModel *tree_model,
 
 static gboolean
 keyword_model_get_iter (GtkTreeModel *tree_model,
-              GtkTreeIter  *iter,
-              GtkTreePath  *path)
+			GtkTreeIter  *iter,
+			GtkTreePath  *path)
 {
         DhKeywordModel     *model;
         DhKeywordModelPriv *priv;
-        GList               *node;
-        gint                 i;
+        GList              *node;
+        const gint         *indices;
         
         g_return_val_if_fail (DH_IS_KEYWORD_MODEL (tree_model), FALSE);
         g_return_val_if_fail (gtk_tree_path_get_depth (path) > 0, FALSE);
@@ -214,13 +221,17 @@ keyword_model_get_iter (GtkTreeModel *tree_model,
         model = DH_KEYWORD_MODEL (tree_model);
         priv  = model->priv;
         
-        i = gtk_tree_path_get_indices (path)[0];
+        indices = gtk_tree_path_get_indices (path);
         
-        if (i >= g_list_length (priv->keyword_words)) {
+        if (indices == NULL) {
                 return FALSE;
         }
         
-        node = g_list_nth (priv->keyword_words, i);
+        if (indices[0] >= g_list_length (priv->keyword_words)) {
+                return FALSE;
+        }
+        
+        node = g_list_nth (priv->keyword_words, indices[0]);
         
         iter->stamp     = priv->stamp;
         iter->user_data = node;
@@ -230,12 +241,11 @@ keyword_model_get_iter (GtkTreeModel *tree_model,
 
 static GtkTreePath *
 keyword_model_get_path (GtkTreeModel *tree_model,
-              GtkTreeIter  *iter)
+			GtkTreeIter  *iter)
 {
         DhKeywordModel     *model = DH_KEYWORD_MODEL (tree_model);
         DhKeywordModelPriv *priv;
         GtkTreePath         *path;
-        GList               *node;
         gint                 i = 0;
 
         g_return_val_if_fail (DH_IS_KEYWORD_MODEL (tree_model), NULL);
@@ -243,18 +253,14 @@ keyword_model_get_path (GtkTreeModel *tree_model,
 
         priv = model->priv;
 
-        for (node = priv->keyword_words; node; node = node->next)
-        {
-                if ((gpointer)node->data == (gpointer)iter->user_data)
-                        break;
-                i++;
-        }
+        i = g_list_position (priv->keyword_words, iter->user_data);
 
-        if (node == NULL) {
+        if (i < 0) {
                 return NULL;
         }
         
         path = gtk_tree_path_new ();
+
         gtk_tree_path_append_index (path, i);
 
         return path;
@@ -262,9 +268,9 @@ keyword_model_get_path (GtkTreeModel *tree_model,
 
 static void
 keyword_model_get_value (GtkTreeModel *tree_model,
-		       GtkTreeIter  *iter,
-		       gint          column,
-		       GValue       *value)
+			 GtkTreeIter  *iter,
+			 gint          column,
+			 GValue       *value)
 {
         DhLink *link;
 	
@@ -302,8 +308,8 @@ keyword_model_iter_next (GtkTreeModel *tree_model, GtkTreeIter *iter)
 
 static gboolean
 keyword_model_iter_children (GtkTreeModel *tree_model,
-                   GtkTreeIter  *iter,
-                   GtkTreeIter  *parent)
+			     GtkTreeIter  *iter,
+			     GtkTreeIter  *parent)
 {
         DhKeywordModelPriv *priv;
         
@@ -331,14 +337,14 @@ keyword_model_iter_children (GtkTreeModel *tree_model,
 
 static gboolean
 keyword_model_iter_has_child (GtkTreeModel *tree_model,
-                    GtkTreeIter  *iter)
+			      GtkTreeIter  *iter)
 {
         return FALSE;
 }
 
 static gint
 keyword_model_iter_n_children (GtkTreeModel *tree_model,
-                     GtkTreeIter  *iter)
+			       GtkTreeIter  *iter)
 {
         DhKeywordModelPriv *priv;
 
@@ -357,9 +363,9 @@ keyword_model_iter_n_children (GtkTreeModel *tree_model,
 
 static gboolean
 keyword_model_iter_nth_child (GtkTreeModel *tree_model,
-                    GtkTreeIter  *iter,
-                    GtkTreeIter  *parent,
-                    gint          n)
+			      GtkTreeIter  *iter,
+			      GtkTreeIter  *parent,
+			      gint          n)
 {
         DhKeywordModelPriv *priv;
         GList               *child;
@@ -385,8 +391,8 @@ keyword_model_iter_nth_child (GtkTreeModel *tree_model,
 
 static gboolean
 keyword_model_iter_parent (GtkTreeModel *tree_model,
-                 GtkTreeIter  *iter,
-                 GtkTreeIter  *child)
+			   GtkTreeIter  *iter,
+			   GtkTreeIter  *child)
 {
         return FALSE;
 }
@@ -409,7 +415,8 @@ dh_keyword_model_set_words (DhKeywordModel *model, GList *keyword_words)
 	g_return_if_fail (DH_IS_KEYWORD_MODEL (model));
 
 	priv = model->priv;
-		
+
+	g_list_free (priv->original_list);
 	priv->original_list = g_list_copy (keyword_words);
 }
 
@@ -423,10 +430,12 @@ dh_keyword_model_filter (DhKeywordModel *model, const gchar *string)
 	gint                 new_length, old_length;
 	gint                 i;
 	GtkTreePath         *path;
- 	GtkTreeIter          iter;
+	GtkTreeIter          iter;
 	gint                 hits = 0;
 	DhLink              *exactlink = NULL;
-	gboolean	     found;
+	gboolean             found;
+	gboolean             case_sensitive;
+	gchar               *lower, *name;
 	gchar              **stringv;
 
 	g_return_val_if_fail (DH_IS_KEYWORD_MODEL (model), NULL);
@@ -443,7 +452,20 @@ dh_keyword_model_filter (DhKeywordModel *model, const gchar *string)
 	if (!strcmp ("", string)) {
 		new_list = NULL;
 	} else {
-	        stringv = g_strsplit (string, " ", -1);
+		stringv = g_strsplit (string, " ", -1);
+
+		/* determine wether or not we should search with case
+		   sensitivity, searches are case sensitive when upper
+		   case is used in the search terms, matching vim
+		   smartcase behaviour */
+		case_sensitive = FALSE;
+		for (i = 0; stringv[i] != NULL; i++) {
+			lower = g_ascii_strdown (stringv[i], -1);
+			if (strcmp (lower, stringv[i])) {
+				case_sensitive = TRUE;
+			}
+			g_free (lower);
+		}
 
 		for (node = priv->original_list; 
 		     node && hits < MAX_HITS; 
@@ -453,11 +475,21 @@ dh_keyword_model_filter (DhKeywordModel *model, const gchar *string)
 			
 			found = TRUE;
 			for (i = 0; stringv[i] != NULL; i++) {
-                                if (!g_strrstr (link->name, stringv[i])) {
-	    				found = FALSE;
+
+				if (!case_sensitive) {
+					name = g_ascii_strdown (link->name, -1);
+				} else {
+					name = g_strdup (link->name);
+				}
+				
+				if (!g_strrstr (name, stringv[i])) {
+					found = FALSE;
+					g_free (name);
 					break;
 				}
-                        }
+
+				g_free (name);
+			}
 				
 			if (found) {
 				/* Include in the new list */
